@@ -27,7 +27,7 @@
 #include <math.h>
 #define COBJMACROS
 #include "d2d1_2.h"
-#include "d3d11.h"
+#include "d3d11_1.h"
 #ifdef D2D1_INIT_GUID
 #include "initguid.h"
 #endif
@@ -73,8 +73,8 @@ struct d2d_error_state
 
 struct d2d_shape_resources
 {
-    ID3D10InputLayout *il;
-    ID3D10VertexShader *vs;
+    ID3D11InputLayout *il;
+    ID3D11VertexShader *vs;
 };
 
 struct d2d_brush_cb
@@ -121,11 +121,33 @@ struct d2d_ps_cb
     struct d2d_brush_cb opacity_brush;
 };
 
+struct d2d_vec4
+{
+    float x, y, z, w;
+};
+
+struct d2d_vs_cb
+{
+    struct
+    {
+        float _11, _21, _31, pad0;
+        float _12, _22, _32, stroke_width;
+    } transform_geometry;
+    struct d2d_vec4 transform_rtx;
+    struct d2d_vec4 transform_rty;
+};
+
 struct d2d_device_context_ops
 {
     HRESULT (*device_context_present)(IUnknown *outer_unknown);
 };
 
+enum d2d_device_context_sampler_limits
+{
+    D2D_SAMPLER_INTERPOLATION_MODE_COUNT = 2,
+    D2D_SAMPLER_EXTEND_MODE_COUNT = 3,
+};
+
 struct d2d_device_context
 {
     ID2D1DeviceContext ID2D1DeviceContext_iface;
@@ -139,16 +161,22 @@ struct d2d_device_context
 
     ID2D1Factory *factory;
     ID2D1Device *device;
-    ID3D10Device *d3d_device;
+    ID3D11Device1 *d3d_device;
+    ID3DDeviceContextState *d3d_state;
     struct d2d_bitmap *target;
-    ID3D10StateBlock *stateblock;
     struct d2d_shape_resources shape_resources[D2D_SHAPE_TYPE_COUNT];
-    ID3D10PixelShader *ps;
-    ID3D10Buffer *ib;
+    ID3D11Buffer *vs_cb;
+    ID3D11PixelShader *ps;
+    ID3D11Buffer *ps_cb;
+    ID3D11Buffer *ib;
     unsigned int vb_stride;
-    ID3D10Buffer *vb;
-    ID3D10RasterizerState *rs;
-    ID3D10BlendState *bs;
+    ID3D11Buffer *vb;
+    ID3D11RasterizerState *rs;
+    ID3D11BlendState *bs;
+    ID3D11SamplerState *sampler_states
+            [D2D_SAMPLER_INTERPOLATION_MODE_COUNT]
+            [D2D_SAMPLER_EXTEND_MODE_COUNT]
+            [D2D_SAMPLER_EXTEND_MODE_COUNT];
 
     struct d2d_error_state error;
     D2D1_DRAWING_STATE_DESCRIPTION1 drawing_state;
@@ -243,12 +271,12 @@ struct d2d_gradient
     LONG refcount;
 
     ID2D1Factory *factory;
-    ID3D10ShaderResourceView *view;
+    ID3D11ShaderResourceView *view;
     D2D1_GRADIENT_STOP *stops;
     UINT32 stop_count;
 };
 
-HRESULT d2d_gradient_create(ID2D1Factory *factory, ID3D10Device *device, const D2D1_GRADIENT_STOP *stops,
+HRESULT d2d_gradient_create(ID2D1Factory *factory, ID3D11Device1 *device, const D2D1_GRADIENT_STOP *stops,
         UINT32 stop_count, D2D1_GAMMA gamma, D2D1_EXTEND_MODE extend_mode,
         struct d2d_gradient **gradient) DECLSPEC_HIDDEN;
 
@@ -287,7 +315,6 @@ struct d2d_brush
             D2D1_EXTEND_MODE extend_mode_x;
             D2D1_EXTEND_MODE extend_mode_y;
             D2D1_INTERPOLATION_MODE interpolation_mode;
-            ID3D10SamplerState *sampler_state;
         } bitmap;
     } u;
 };
@@ -303,9 +330,9 @@ HRESULT d2d_radial_gradient_brush_create(ID2D1Factory *factory,
 HRESULT d2d_bitmap_brush_create(ID2D1Factory *factory, ID2D1Bitmap *bitmap,
         const D2D1_BITMAP_BRUSH_PROPERTIES1 *bitmap_brush_desc, const D2D1_BRUSH_PROPERTIES *brush_desc,
         struct d2d_brush **brush) DECLSPEC_HIDDEN;
-void d2d_brush_bind_resources(struct d2d_brush *brush, ID3D10Device *device, unsigned int brush_idx) DECLSPEC_HIDDEN;
-HRESULT d2d_brush_get_ps_cb(struct d2d_brush *brush, struct d2d_brush *opacity_brush, BOOL outline, BOOL is_arc,
-        struct d2d_device_context *render_target, ID3D10Buffer **ps_cb) DECLSPEC_HIDDEN;
+void d2d_brush_bind_resources(struct d2d_brush *brush, struct d2d_device_context *context,
+        unsigned int brush_idx) DECLSPEC_HIDDEN;
+BOOL d2d_brush_fill_cb(const struct d2d_brush *brush, struct d2d_brush_cb *cb) DECLSPEC_HIDDEN;
 struct d2d_brush *unsafe_impl_from_ID2D1Brush(ID2D1Brush *iface) DECLSPEC_HIDDEN;
 
 struct d2d_stroke_style
@@ -349,10 +376,10 @@ struct d2d_bitmap
     LONG refcount;
 
     ID2D1Factory *factory;
-    ID3D10ShaderResourceView *srv;
-    ID3D10RenderTargetView *rtv;
+    ID3D11ShaderResourceView *srv;
+    ID3D11RenderTargetView *rtv;
     IDXGISurface *surface;
-    ID3D10Resource *resource;
+    ID3D11Resource *resource;
     D2D1_SIZE_U pixel_size;
     D2D1_PIXEL_FORMAT format;
     float dpi_x;
@@ -405,11 +432,6 @@ struct d2d_face
     UINT16 v[3];
 };
 
-struct d2d_vec4
-{
-    float x, y, z, w;
-};
-
 struct d2d_outline_vertex
 {
     D2D1_POINT_2F position;
@@ -543,13 +565,29 @@ struct d2d_device
 
 void d2d_device_init(struct d2d_device *device, ID2D1Factory1 *factory, IDXGIDevice *dxgi_device) DECLSPEC_HIDDEN;
 
+struct d2d_effect_info
+{
+    const CLSID *clsid;
+    UINT32 default_input_count;
+    UINT32 min_inputs;
+    UINT32 max_inputs;
+};
+
 struct d2d_effect
 {
     ID2D1Effect ID2D1Effect_iface;
+    ID2D1Image ID2D1Image_iface;
     LONG refcount;
+
+    const struct d2d_effect_info *info;
+
+    ID2D1Factory *factory;
+    ID2D1Image **inputs;
+    size_t inputs_size;
+    size_t input_count;
 };
 
-void d2d_effect_init(struct d2d_effect *effect) DECLSPEC_HIDDEN;
+HRESULT d2d_effect_init(struct d2d_effect *effect, ID2D1Factory *factory, const CLSID *effect_id) DECLSPEC_HIDDEN;
 
 static inline BOOL d2d_array_reserve(void **elements, size_t *capacity, size_t count, size_t size)
 {
@@ -640,6 +678,13 @@ static inline D2D1_INTERPOLATION_MODE d2d1_1_interp_mode_from_d2d1(D2D1_BITMAP_I
     return (D2D1_INTERPOLATION_MODE)mode;
 }
 
+static inline const char *debug_d2d_color_f(const D2D1_COLOR_F *colour)
+{
+    if (!colour)
+        return "(null)";
+    return wine_dbg_sprintf("{%.8e, %.8e, %.8e, %.8e}", colour->r, colour->g, colour->b, colour->a);
+}
+
 static inline const char *debug_d2d_point_2f(const D2D1_POINT_2F *point)
 {
     if (!point)
