@@ -22,15 +22,11 @@
 
 #define COBJMACROS
 
-#include "config.h"
-
 #include <stdarg.h>
-#ifdef HAVE_LIBXML2
-# include <libxml/parser.h>
-# include <libxml/xmlerror.h>
-# include <libxml/xpath.h>
-# include <libxml/xpathInternals.h>
-#endif
+#include <libxml/parser.h>
+#include <libxml/xmlerror.h>
+#include <libxml/xpath.h>
+#include <libxml/xpathInternals.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -52,8 +48,6 @@
  *
  */
 
-#ifdef HAVE_LIBXML2
-
 WINE_DEFAULT_DEBUG_CHANNEL(msxml);
 
 int registerNamespaces(xmlXPathContextPtr ctxt);
@@ -838,5 +832,3 @@ cleanup:
     xmlXPathFreeContext(ctxt);
     return hr;
 }
-
-#endif
