@@ -23,16 +23,20 @@
 #endif
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <signal.h>
-#ifdef HAVE_LINK_H
-# include <link.h>
-#endif
+#include <math.h>
+#include <string.h>
+#include <stdlib.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <dlfcn.h>
 #ifdef HAVE_PWD_H
 # include <pwd.h>
 #endif
@@ -344,6 +348,7 @@ static void * const syscalls[] =
     NtYieldExecution,
     __wine_dbg_write,
     __wine_unix_call,
+    __wine_unix_spawnvp,
     wine_nt_to_unix_file_name,
     wine_server_call,
     wine_server_fd_to_handle,
