@@ -23,7 +23,6 @@
 #endif
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -41,6 +40,7 @@
 #include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #ifdef HAVE_SYS_SOCKET_H
@@ -58,9 +58,7 @@
 #ifdef HAVE_SYS_PRCTL_H
 # include <sys/prctl.h>
 #endif
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
+#include <sys/stat.h>
 #ifdef HAVE_SYS_SYSCALL_H
 # include <sys/syscall.h>
 #endif
@@ -70,9 +68,7 @@
 #ifdef HAVE_SYS_THR_H
 #include <sys/thr.h>
 #endif
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#include <unistd.h>
 #ifdef __APPLE__
 #include <crt_externs.h>
 #include <spawn.h>
@@ -378,15 +374,19 @@ static void invoke_system_apc( const apc_call_t *call, apc_result_t *result, BOO
     case APC_ASYNC_IO:
     {
         struct async_fileio *user = wine_server_get_ptr( call->async_io.user );
-        ULONG_PTR info = 0;
+        ULONG_PTR info = call->async_io.result;
+        NTSTATUS status;
 
         result->type = call->type;
-        result->async_io.status = user->callback( user, &info, call->async_io.status );
-        if (result->async_io.status != STATUS_PENDING)
+        status = call->async_io.status;
+        if (user->callback( user, &info, &status ))
         {
+            result->async_io.status = status;
             result->async_io.total = info;
+            /* the server will pass us NULL if a call failed synchronously */
             set_async_iosb( call->async_io.sb, result->async_io.status, info );
         }
+        else result->async_io.status = STATUS_PENDING; /* restart it */
         break;
     }
     case APC_VIRTUAL_ALLOC:
