@@ -1424,6 +1424,7 @@ static void test_Win32_OperatingSystem( IWbemServices *services )
     check_property( obj, L"Version", VT_BSTR, CIM_STRING );
     check_property( obj, L"TotalVisibleMemorySize", VT_BSTR, CIM_UINT64 );
     check_property( obj, L"TotalVirtualMemorySize", VT_BSTR, CIM_UINT64 );
+    check_property( obj, L"Status", VT_BSTR, CIM_STRING );
     check_property( obj, L"SystemDrive", VT_BSTR, CIM_STRING );
 
     IWbemClassObject_Release( obj );
@@ -1642,6 +1643,7 @@ static void test_Win32_VideoController( IWbemServices *services )
         if (hr != S_OK) break;
 
         check_property( obj, L"__CLASS", VT_BSTR, CIM_STRING );
+        check_property( obj, L"__DERIVATION", VT_BSTR | VT_ARRAY, CIM_STRING | CIM_FLAG_ARRAY );
         check_property( obj, L"__GENUS", VT_I4, CIM_SINT32 );
         check_property( obj, L"__NAMESPACE", VT_BSTR, CIM_STRING );
         check_property( obj, L"__PATH", VT_BSTR, CIM_STRING );
