@@ -1067,106 +1067,64 @@ static struct unix_funcs unix_funcs =
 {
     NtGdiAbortDoc,
     NtGdiAbortPath,
-    NtGdiAddFontMemResourceEx,
-    NtGdiAddFontResourceW,
     NtGdiAlphaBlend,
     NtGdiAngleArc,
     NtGdiArcInternal,
     NtGdiBeginPath,
     NtGdiBitBlt,
     NtGdiCloseFigure,
-    NtGdiCombineRgn,
     NtGdiComputeXformCoefficients,
-    NtGdiCreateBitmap,
-    NtGdiCreateClientObj,
     NtGdiCreateCompatibleBitmap,
     NtGdiCreateCompatibleDC,
-    NtGdiCreateDIBSection,
     NtGdiCreateDIBitmapInternal,
-    NtGdiCreateEllipticRgn,
-    NtGdiCreateHalftonePalette,
     NtGdiCreateMetafileDC,
-    NtGdiCreatePaletteInternal,
-    NtGdiCreatePen,
-    NtGdiCreateRectRgn,
-    NtGdiCreateRoundRectRgn,
     NtGdiDdDDICheckVidPnExclusiveOwnership,
-    NtGdiDdDDICloseAdapter,
     NtGdiDdDDICreateDCFromMemory,
-    NtGdiDdDDICreateDevice,
     NtGdiDdDDIDestroyDCFromMemory,
     NtGdiDdDDIDestroyDevice,
     NtGdiDdDDIEscape,
-    NtGdiDdDDIOpenAdapterFromDeviceName,
-    NtGdiDdDDIOpenAdapterFromHdc,
-    NtGdiDdDDIOpenAdapterFromLuid,
-    NtGdiDdDDIQueryStatistics,
-    NtGdiDdDDISetQueuedLimit,
     NtGdiDdDDISetVidPnSourceOwner,
-    NtGdiDeleteClientObj,
     NtGdiDeleteObjectApp,
-    NtGdiDescribePixelFormat,
     NtGdiDoPalette,
-    NtGdiDrawStream,
     NtGdiEllipse,
     NtGdiEndDoc,
     NtGdiEndPath,
     NtGdiEndPage,
     NtGdiEnumFonts,
-    NtGdiEqualRgn,
     NtGdiExcludeClipRect,
-    NtGdiExtCreatePen,
     NtGdiExtEscape,
     NtGdiExtFloodFill,
     NtGdiExtTextOutW,
-    NtGdiExtCreateRegion,
-    NtGdiExtGetObjectW,
     NtGdiExtSelectClipRgn,
     NtGdiFillPath,
     NtGdiFillRgn,
-    NtGdiFlattenPath,
     NtGdiFontIsLinked,
     NtGdiFrameRgn,
     NtGdiGetAndSetDCDword,
     NtGdiGetAppClipBox,
-    NtGdiGetBitmapBits,
-    NtGdiGetBitmapDimension,
     NtGdiGetBoundsRect,
     NtGdiGetCharABCWidthsW,
     NtGdiGetCharWidthW,
     NtGdiGetCharWidthInfo,
-    NtGdiGetColorAdjustment,
-    NtGdiGetDCObject,
     NtGdiGetDIBitsInternal,
     NtGdiGetDeviceCaps,
     NtGdiGetDeviceGammaRamp,
     NtGdiGetFontData,
-    NtGdiGetFontFileData,
-    NtGdiGetFontFileInfo,
     NtGdiGetFontUnicodeRanges,
     NtGdiGetGlyphIndicesW,
     NtGdiGetGlyphOutline,
     NtGdiGetKerningPairs,
     NtGdiGetNearestColor,
-    NtGdiGetNearestPaletteIndex,
     NtGdiGetOutlineTextMetricsInternalW,
-    NtGdiGetPath,
     NtGdiGetPixel,
     NtGdiGetRandomRgn,
     NtGdiGetRasterizerCaps,
     NtGdiGetRealizationInfo,
-    NtGdiGetRegionData,
-    NtGdiGetRgnBox,
-    NtGdiGetSpoolMessage,
-    NtGdiGetSystemPaletteUse,
     NtGdiGetTextCharsetInfo,
     NtGdiGetTextExtentExW,
     NtGdiGetTextFaceW,
     NtGdiGetTextMetricsW,
-    NtGdiGetTransform,
     NtGdiGradientFill,
-    NtGdiHfontCreate,
-    NtGdiInitSpool,
     NtGdiIntersectClipRect,
     NtGdiInvertRgn,
     NtGdiLineTo,
@@ -1174,25 +1132,18 @@ static struct unix_funcs unix_funcs =
     NtGdiModifyWorldTransform,
     NtGdiMoveTo,
     NtGdiOffsetClipRgn,
-    NtGdiOffsetRgn,
     NtGdiOpenDCW,
     NtGdiPatBlt,
-    NtGdiPathToRegion,
     NtGdiPlgBlt,
     NtGdiPolyDraw,
     NtGdiPolyPolyDraw,
-    NtGdiPtInRegion,
     NtGdiPtVisible,
-    NtGdiRectInRegion,
     NtGdiRectVisible,
     NtGdiRectangle,
-    NtGdiRemoveFontMemResourceEx,
-    NtGdiRemoveFontResourceW,
     NtGdiResetDC,
     NtGdiResizePalette,
     NtGdiRestoreDC,
     NtGdiRoundRect,
-    NtGdiSaveDC,
     NtGdiScaleViewportExtEx,
     NtGdiScaleWindowExtEx,
     NtGdiSelectBitmap,
@@ -1200,31 +1151,19 @@ static struct unix_funcs unix_funcs =
     NtGdiSelectClipPath,
     NtGdiSelectFont,
     NtGdiSelectPen,
-    NtGdiSetBitmapBits,
-    NtGdiSetBitmapDimension,
-    NtGdiSetBrushOrg,
     NtGdiSetBoundsRect,
-    NtGdiSetColorAdjustment,
     NtGdiSetDIBitsToDeviceInternal,
     NtGdiSetDeviceGammaRamp,
     NtGdiSetLayout,
-    NtGdiSetMagicColors,
-    NtGdiSetMetaRgn,
     NtGdiSetPixel,
-    NtGdiSetPixelFormat,
-    NtGdiSetRectRgn,
     NtGdiSetSystemPaletteUse,
-    NtGdiSetTextJustification,
-    NtGdiSetVirtualResolution,
     NtGdiStartDoc,
     NtGdiStartPage,
     NtGdiStretchBlt,
     NtGdiStretchDIBitsInternal,
     NtGdiStrokeAndFillPath,
     NtGdiStrokePath,
-    NtGdiSwapBuffers,
     NtGdiTransparentBlt,
-    NtGdiTransformPoints,
     NtGdiUnrealizeObject,
     NtGdiUpdateColors,
     NtGdiWidenPath,
