@@ -431,9 +431,9 @@ void state_unbind_resources(struct wined3d_state *state)
 
         for (j = 0; j < MAX_CONSTANT_BUFFERS; ++j)
         {
-            if ((buffer = state->cb[i][j]))
+            if ((buffer = state->cb[i][j].buffer))
             {
-                state->cb[i][j] = NULL;
+                state->cb[i][j].buffer = NULL;
                 wined3d_buffer_decref(buffer);
             }
         }
@@ -585,8 +585,10 @@ ULONG CDECL wined3d_stateblock_decref(struct wined3d_stateblock *stateblock)
 
     if (!refcount)
     {
+        wined3d_mutex_lock();
         wined3d_stateblock_state_cleanup(&stateblock->stateblock_state);
         heap_free(stateblock);
+        wined3d_mutex_unlock();
     }
 
     return refcount;
@@ -1204,8 +1206,7 @@ HRESULT CDECL wined3d_stateblock_set_vs_consts_f(struct wined3d_stateblock *stat
     TRACE("stateblock %p, start_idx %u, count %u, constants %p.\n",
             stateblock, start_idx, count, constants);
 
-    if (!constants || start_idx >= d3d_info->limits.vs_uniform_count
-            || count > d3d_info->limits.vs_uniform_count - start_idx)
+    if (!constants || !wined3d_bound_range(start_idx, count, d3d_info->limits.vs_uniform_count))
         return WINED3DERR_INVALIDCALL;
 
     memcpy(&stateblock->stateblock_state.vs_consts_f[start_idx], constants, count * sizeof(*constants));
@@ -1273,8 +1274,7 @@ HRESULT CDECL wined3d_stateblock_set_ps_consts_f(struct wined3d_stateblock *stat
     TRACE("stateblock %p, start_idx %u, count %u, constants %p.\n",
             stateblock, start_idx, count, constants);
 
-    if (!constants || start_idx >= d3d_info->limits.ps_uniform_count
-            || count > d3d_info->limits.ps_uniform_count - start_idx)
+    if (!constants || !wined3d_bound_range(start_idx, count, d3d_info->limits.ps_uniform_count))
         return WINED3DERR_INVALIDCALL;
 
     memcpy(&stateblock->stateblock_state.ps_consts_f[start_idx], constants, count * sizeof(*constants));
@@ -1831,8 +1831,8 @@ static void init_default_sampler_states(DWORD states[WINED3D_MAX_COMBINED_SAMPLE
 
 static void state_init_default(struct wined3d_state *state, const struct wined3d_d3d_info *d3d_info)
 {
-    unsigned int i;
     struct wined3d_matrix identity;
+    unsigned int i, j;
 
     TRACE("state %p, d3d_info %p.\n", state, d3d_info);
 
@@ -1869,6 +1869,12 @@ static void state_init_default(struct wined3d_state *state, const struct wined3d
 
     for (i = 0; i < WINED3D_MAX_STREAMS; ++i)
         state->streams[i].frequency = 1;
+
+    for (i = 0; i < WINED3D_SHADER_TYPE_COUNT; ++i)
+    {
+        for (j = 0; j < MAX_CONSTANT_BUFFERS; ++j)
+            state->cb[i][j].size = WINED3D_MAX_CONSTANT_BUFFER_SIZE * 16;
+    }
 }
 
 void state_init(struct wined3d_state *state, const struct wined3d_d3d_info *d3d_info,
