@@ -19,7 +19,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 #include "wine/asm.h"
 
 #ifdef __ASM_OBSOLETE
@@ -29,10 +28,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#include <unistd.h>
+#include <dlfcn.h>
 #ifdef HAVE_PWD_H
 #include <pwd.h>
 #endif
@@ -109,7 +108,6 @@ static char *build_path( const char *dir, const char *name )
 /* return the directory that contains the library at run-time */
 static char *get_runtime_libdir(void)
 {
-#ifdef HAVE_DLADDR
     Dl_info info;
     char *libdir;
 
@@ -123,7 +121,6 @@ static char *get_runtime_libdir(void)
         libdir[len] = 0;
         return libdir;
     }
-#endif /* HAVE_DLADDR */
     return NULL;
 }
 
